<?php

namespace Modules\Fornecimento\Modules\IntegracaoAF\Services;

use Util\SupUtilities;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\Report\ReportService;

class IntegracaoAFService {
    
    const NAME_REPORT = "SUP00810";
    const STRATEGY_REPORT = "BIRT";
    
    protected $supUtilities;
    protected $environment;
    protected $entityManager;
    protected $reportService;
    protected $smtpUtil;
    
    protected $paramIntegra = "SELECT P.* FROM USR_PARAMINTEGRA P, FORNECEDOR F WHERE P.CDFORNECED = F.CDFORNECED AND NRINSJURFORN = :NRINSJURFORN";
    protected $afImportacao = "SELECT A.CDFORNECED, F.NRINSJURFORN, A.USR_NRIMPORTAF, A.CDFILDEST, A.CDFILLANC, A.NRAUTOFORN, A.NRORG FROM AUTOFORNE A, FORNECEDOR F WHERE A.CDFORNECED = F.CDFORNECED AND A.NRORG = F.NRORG AND A.USR_NRIMPORTAF = :USR_NRIMPORTAF AND F.NRINSJURFORN = :NRINSJURFORN AND A.CDFILLANC = :CDFILLANC AND A.CDFILDEST = :CDFILDEST";
    protected $fornecedor = "SELECT CDFORNECED, CDFORMPGTO, IDTPFORPGTFO, NMFANTFORN, NRPARCPGTOFO, NRDIAPGFIXFO, IDUTDIAPGTFO FROM FORNECEDOR WHERE NRINSJURFORN = :NRINSJURFORN";
    protected $unidade = "SELECT CDFILIAL FROM FILIAL WHERE CDFILIAL = :CDFILIAL";
    protected $empresa = "SELECT CDEMPRESA FROM EMPRESA WHERE NRINSJUREMPR = :NRINSJUREMPR";
    protected $operador = "SELECT NMOPERADOR, DSEMAILOPER FROM OPERADOR WHERE CDOPERADOR = :CDOPERADOR";
    protected $produto = "SELECT CDPRODUTO, CDARVPROD, NMPRODUTO FROM PRODUTO WHERE CDPRODUTO = :CDPRODUTO";
    protected $prodForn = "SELECT MAX(CDPRODCOMPL) CDPRODCOMPL FROM PRODFORNCOMP P, FORNECEDOR F WHERE P.CDFORNECED = F.CDFORNECED AND P.NRORG = F.NRORG AND F.NRINSJURFORN = :NRINSJURFORN AND :CDPRODUTO LIKE P.CDPRODUTO||'%'";
    protected $centralCmp = "SELECT NVL(CDFILCENTCMP, CDFILIAL) AS CDFILCENTCMP FROM FILIAL WHERE CDFILIAL = :CDFILIAL";
    protected $melhorPreco = "SELECT FILIAL.CDFILIAL, NVL(ITEMCOLE.VRPRECUNIITM, 0) VRPRECUNIITM FROM ITEMCOLE, FILIAL, PRODUTO, FORNECEDOR WHERE ITEMCOLE.CDFILIAL = :CDCENTCMP AND FORNECEDOR.NRINSJURFORN = :NRINSJURFORN AND FILIAL.CDFILIAL = :CDFILDEST AND PRODUTO.CDPRODUTO = :CDPRODUTO AND ITEMCOLE.CDREGIAO = FILIAL.CDREGIAO AND ITEMCOLE.CDPRODUTO = PRODUTO.CDPRODUTO AND ITEMCOLE.CDFORNECED = FORNECEDOR.CDFORNECED AND NVL(ITEMCOLE.VRPRECUNIITM, 0) > 0 AND TO_DATE(:DTPROGENTAF, 'DD/MM/YYYY') BETWEEN ITEMCOLE.DTINIVALITM AND ITEMCOLE.DTFINVALITM";
    
    protected $insertAutoforne = "INSERT INTO AUTOFORNE(CDFILDEST, CDFILLANC, NRAUTOFORN, CDFORNECED, DTAUTOFORN, CDOPERADOR, CDFORMPGTO, IDTPFORPGTAF, IDTIPOEMITAF, CDEMPRESA, NRPARCPGTOAF, NRDIAPGFIXAF, IDUTDIAPGTAF, IDTIPOLANCAF, USR_NRIMPORTAF) VALUES (:CDFILDEST, :CDFILLANC, :NRAUTOFORN, :CDFORNECED, TRUNC(SYSDATE), :CDOPERADOR, :CDFORMPGTO, :IDTPFORPGTAF, '1', :CDEMPRESA, :NRPARCPGTOAF, :NRDIAPGFIXAF, :IDUTDIAPGTAF, :IDTIPOLANCAF, :USR_NRIMPORTAF)";
    protected $insertPrazopgtaf = "INSERT INTO PRAZOPGTAF(CDFILDEST, CDFILLANC, NRAUTOFORN, NRDIASPGTOAF, VRPERCPGTOAF) (SELECT :CDFILDEST, :CDFILLANC, :NRAUTOFORN, NRDIASPGTOFO, VRPERCPGTOFO FROM PRAZOPGTFORN WHERE CDFORNECED = :CDFORNECED)";
    protected $insertItemautf = "INSERT INTO ITEMAUTF(CDFILDEST, CDFILLANC, NRAUTOFORN, CDPRODUTO, QTPROGENTAF, VRPREUNIAF, DTPROGENTAF, CDMARCA) SELECT :CDFILDEST, :CDFILLANC, :NRAUTOFORN, :CDPRODUTO, :QTPROGENTAF, :VRPREUNIAF, TO_DATE(:DTPROGENTAF, 'DD/MM/YYYY'), :CDMARCA FROM DUAL WHERE NOT EXISTS(SELECT 1 FROM ITEMAUTF I WHERE I.CDFILDEST = :CDFILDEST AND I.CDFILLANC = :CDFILLANC AND I.NRAUTOFORN = :NRAUTOFORN AND I.CDPRODUTO = :CDPRODUTO AND I.DTPROGENTAF = TO_DATE(:DTPROGENTAF, 'DD/MM/YYYY') AND I.CDMARCA = :CDMARCA)";
    protected $insertMensagem = "INSERT INTO MENSAUTF (NRMENSAUTFOR, CDFILLANC, NRAUTOFORN, DSMENSAF, CDFILDEST) VALUES (:NRMENSAUTFOR, :CDFILLANC, :NRAUTOFORN, :DSMENSAF, :CDFILDEST)";
    protected $insertLog = "INSERT INTO LOGIMPITEMAUTF(USR_NRIMPORTAF, CDFILDEST, CDPRODUTO, DTPROGENTAF, DTIMPITEMAUTF, DSERROIMPORTAF) VALUES (:USR_NRIMPORTAF, :CDFILDEST, :CDPRODUTO, TO_DATE(:DTPROGENTAF, 'DD/MM/YYYY'), SYSDATE, :DSERROIMPORTAF)";
    
    protected $deleteLog = "DELETE FROM LOGIMPITEMAUTF WHERE USR_NRIMPORTAF = :USR_NRIMPORTAF AND CDFILDEST = :CDFILDEST AND TO_CHAR(DTPROGENTAF, 'DD/MM/YYYY') = :DTPROGENTAF";
    
    public function __construct(SupUtilities $supUtilities, $environment, $entityManager, $reportService, $smtpUtil) {
        $this->supUtilities = $supUtilities;
        $this->environment = $environment;
        $this->connection = $entityManager->getConnection();
        $this->reportService = $reportService;
        $this->smtpUtil = $smtpUtil;
    }
    
    public function iniciaTransacao(){
        $this->supUtilities->iniciaTransacao();
    }

    public function commit(){
        $this->supUtilities->commit();
    }

    public function flush(){
        $this->supUtilities->flush();
    }

    public function rollback(){
        $this->supUtilities->rollback();
    }
    
    public function retornaCentralCompras($cdfilial) {
        
        $param = [
            "CDFILIAL" => $cdfilial
        ];
        
        $centralCmp = $this->supUtilities->crud($this->centralCmp, $param);

        return count($centralCmp) ? $centralCmp['0']['CDFILCENTCMP'] : false;
        
    }
    
    public function retornaUsrParamIntegra($nrinsjurforn) {
        
        $param = [
            "NRINSJURFORN" => $nrinsjurforn
        ];
        
        $fornecedor = $this->supUtilities->crud($this->paramIntegra, $param);
        
        return $fornecedor;
        
    }
    
    public function retornaAutofornImportacao($nrinsjurforn, $usrnrimportaf, $cdfillanc, $cdfildest) {

        $param = [
            "USR_NRIMPORTAF" => $usrnrimportaf,
            "NRINSJURFORN" => $nrinsjurforn,
            "CDFILLANC" => $cdfillanc,
            "CDFILDEST" => $cdfildest
        ];

        $af = $this->supUtilities->crud($this->afImportacao, $param);
        
        return $af;
        
    }
    
    public function retornaFornecedor($nrinsjurforn) {
        
        $param = [
            "NRINSJURFORN" => $nrinsjurforn
        ];
        
        $fornecedor = $this->supUtilities->crud($this->fornecedor, $param);
        
        return $fornecedor;
        
    }
    
    public function retornaEmpresa($nrinsjurempr) {
        
        $param = [
            "NRINSJUREMPR" => $nrinsjurempr
        ];
        
        $empresa = $this->supUtilities->crud($this->empresa, $param);
        
        return $empresa;
        
    }
    
    public function retornaUnidade($cdfilial) {
        
        $param = [
            "CDFILIAL" => $cdfilial
        ];
        
        $unidade = $this->supUtilities->crud($this->unidade, $param);
        
        return $unidade;
        
    }
    
    public function retornaProduto($cdproduto) {
        
        $param = [
            "CDPRODUTO" => $cdproduto
        ];
        
        $produto = $this->supUtilities->crud($this->produto, $param);
        
        return $produto;
        
    }
    
    public function retornaOperador($cdoperador) {
        
        $param = [
            "CDOPERADOR" => $cdoperador
        ];
        
        $operador = $this->supUtilities->crud($this->operador, $param);
        
        return $operador;
        
    }
    
    public function retornaProdutoFornecedor($nrinsjurforn, $cdproduto) {
        
        $param = [
            "NRINSJURFORN" => $nrinsjurforn,
            "CDPRODUTO" => $cdproduto
        ];
        
        $prodForn = $this->supUtilities->crud($this->prodForn, $param);
        
        return $prodForn;
        
    }
    
    public function retornaMelhorPreco($cdcentcmp, $nrinsjurforn, $cdfildest, $cdproduto, $dtprogentaf) {
        
        $param = [
            "CDCENTCMP" => $cdcentcmp,
            "NRINSJURFORN" => $nrinsjurforn,
            "CDFILDEST" => $cdfildest,
            "CDPRODUTO" => $cdproduto,
            "DTPROGENTAF" => $dtprogentaf
        ];
        
        $melhorPreco = $this->supUtilities->crud($this->melhorPreco, $param);
        
        return count($melhorPreco) ? $melhorPreco['0']['VRPRECUNIITM'] : false;
        
    }
    
    public function geraNovoCodigoSC($cdcontador) {
        
        return $this->supUtilities->generateCodeSC($cdcontador);
        
    }
    
    public function insereAutoforne($cdfillanc, $cdoperador, $idtipolancaf, $cdempresa, $cdfildest, $nrautoforn, $fornecedor, $usrNrimportaf) {
        
        $param = [
            "CDFILDEST" => $cdfildest,
            "CDFILLANC" => $cdfillanc,
            "NRAUTOFORN" => $nrautoforn,
            "USR_NRIMPORTAF" => $usrNrimportaf,
            "CDFORNECED" => $fornecedor["CDFORNECED"],
            "CDOPERADOR" => $cdoperador,
            "CDFORMPGTO" => $fornecedor["CDFORMPGTO"],
            "IDTPFORPGTAF" => $fornecedor["IDTPFORPGTFO"],
            "CDEMPRESA" => $cdempresa,
            "IDTIPOLANCAF" => $idtipolancaf,
            "NRPARCPGTOAF" => $fornecedor["NRPARCPGTOFO"],
            "NRDIAPGFIXAF" => $fornecedor["NRDIAPGFIXFO"],
            "IDUTDIAPGTAF" => $fornecedor["IDUTDIAPGTFO"]
        ];
        
        $this->supUtilities->crud($this->insertAutoforne, $param);
        
    }
    
    public function inserePrazopgtaf($cdfildest, $cdfillanc, $nrautoforn, $cdforneced) {
        
        $param = [
            "CDFILDEST" => $cdfildest,
            "CDFILLANC" => $cdfillanc,
            "NRAUTOFORN" => $nrautoforn,
            "CDFORNECED" => $cdforneced
        ];
        
        $this->supUtilities->crud($this->insertPrazopgtaf, $param);
        
    }
    
    public function insereItemautaf($cdfillanc, $cdfildest, $nrautoforn, $dtprogentaf, $cdmarca, $item) {
        
        $param = [
            "CDFILLANC" => $cdfillanc,
            "CDFILDEST" => $cdfildest,
            "NRAUTOFORN" => $nrautoforn,
            "DTPROGENTAF" => $dtprogentaf,
            "CDPRODUTO" => $item['codCliente'],
            "CDMARCA" => $cdmarca,
            "QTPROGENTAF" => $item['quantidade'],
            "VRPREUNIAF" => $item['vlrUnitario']
        ];
        
        $this->supUtilities->crud($this->insertItemautf, $param);
        
    }
    
    public function insertMensagem($nrmensautfor, $cdfillanc, $nrautoforn, $dsmensaf, $cdfildest) {
        
        $param = [
            "NRMENSAUTFOR" => $nrmensautfor,
            "CDFILLANC" => $cdfillanc,
            "NRAUTOFORN" => $nrautoforn,
            "DSMENSAF" => $dsmensaf,
            "CDFILDEST" => $cdfildest
        ];
        
        $this->supUtilities->crud($this->insertMensagem, $param);
        
    }
    
    public function deleteLog($usrNrimportaf, $cdfildest, $dtprogentaf) {
        
        $param = [
            "USR_NRIMPORTAF" => $usrNrimportaf,
            "CDFILDEST" => $cdfildest,
            "DTPROGENTAF" => $dtprogentaf
        ];
        
        $this->supUtilities->crud($this->deleteLog, $param);
        
    }
    
    public function insertLog($usrNrimportaf, $cdfildest, $dtprogentaf, $cdproduto, $dserroimportaf) {
        
        $param = [
            "USR_NRIMPORTAF" => $usrNrimportaf,
            "CDFILDEST" => $cdfildest,
            "DTPROGENTAF" => $dtprogentaf,
            "CDPRODUTO" => $cdproduto,
            "DSERROIMPORTAF" => $dserroimportaf
        ];
        
        $this->supUtilities->crud($this->insertLog, $param);
        
    }
    
    public function executeProcedureFechamentoAF($cdfildest, $cdfillanc, $nrautoforn) {
        
        $procedure = new StoredProcedure($this->connection, 'FECHAMENTO_AF');
        
        $procedure->addParam(new Param('P_CDFILDEST', Param::PARAM_INPUT, $cdfildest, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILLANC', Param::PARAM_INPUT, $cdfillanc, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAUTOFORN', Param::PARAM_INPUT, $nrautoforn, Param::PARAM_TYPE_STR));
        
        $procedure->execute();
        
    }
    
    public function geraRelatorioErros($usrNrimportaf, $cdfildest) {
        
        $param = [
            'P_CDFILDEST' => $cdfildest,
            'P_TODASDT' => 'T',
            'P_TUSR_NRIMPORTAF' => null,
            'P_USR_NRIMPORTAF' => $usrNrimportaf,
            'P_NRORG' => '1',
            'P_DTFIN' => null,
            'P_DTINI' => null
        ];
        
        $report = $this->reportService->createRemoteReport(self::STRATEGY_REPORT, self::NAME_REPORT, $param);
        
        return $report;
        
    }
    
    public function enviaEmail($assunto, $mensagem, $destinatario) {
        
        $this->smtpUtil->setSubject($assunto);
        $this->smtpUtil->sendMail($mensagem, $destinatario);
        
    }
    
}    